# Generated by roxygen2: do not edit by hand

S3method("[",tbl_spark)
S3method("names<-",tbl_spark)
S3method(arrow_enabled_object,data.frame)
S3method(arrow_enabled_object,default)
S3method(arrow_enabled_object,spark_jobj)
S3method(arrow_enabled_object,tbl_df)
S3method(arrow_enabled_object,tbl_spark)
S3method(augment,"_ml_model_decision_tree_classification")
S3method(augment,"_ml_model_decision_tree_regression")
S3method(augment,"_ml_model_gbt_classification")
S3method(augment,"_ml_model_gbt_regression")
S3method(augment,"_ml_model_linear_regression")
S3method(augment,"_ml_model_logistic_regression")
S3method(augment,"_ml_model_random_forest_classification")
S3method(augment,"_ml_model_random_forest_regression")
S3method(augment,ml_model)
S3method(augment,ml_model_aft_survival_regression)
S3method(augment,ml_model_als)
S3method(augment,ml_model_bisecting_kmeans)
S3method(augment,ml_model_decision_tree_classification)
S3method(augment,ml_model_decision_tree_regression)
S3method(augment,ml_model_gaussian_mixture)
S3method(augment,ml_model_gbt_classification)
S3method(augment,ml_model_gbt_regression)
S3method(augment,ml_model_generalized_linear_regression)
S3method(augment,ml_model_isotonic_regression)
S3method(augment,ml_model_kmeans)
S3method(augment,ml_model_lda)
S3method(augment,ml_model_linear_regression)
S3method(augment,ml_model_linear_svc)
S3method(augment,ml_model_logistic_regression)
S3method(augment,ml_model_multilayer_perceptron_classification)
S3method(augment,ml_model_naive_bayes)
S3method(augment,ml_model_pca)
S3method(augment,ml_model_random_forest_classification)
S3method(augment,ml_model_random_forest_regression)
S3method(cbind,tbl_spark)
S3method(collect,spark_jobj)
S3method(collect,tbl_spark)
S3method(compute,tbl_spark)
S3method(connection_is_open,livy_connection)
S3method(connection_is_open,spark_gateway_connection)
S3method(connection_is_open,spark_shell_connection)
S3method(connection_is_open,spark_worker_connection)
S3method(connection_is_open,test_connection)
S3method(copy_to,spark_connection)
S3method(copy_to,src_spark)
S3method(create_hive_context,livy_connection)
S3method(create_hive_context,test_connection)
S3method(db_connection_describe,src_spark)
S3method(db_data_type,spark_connection)
S3method(dbplyr_edition,spark_connection)
S3method(dim,tbl_spark)
S3method(distinct,tbl_spark)
S3method(do,tbl_spark)
S3method(fill,tbl_spark)
S3method(filter,tbl_spark)
S3method(fitted,ml_model_prediction)
S3method(ft_binarizer,ml_pipeline)
S3method(ft_binarizer,spark_connection)
S3method(ft_binarizer,tbl_spark)
S3method(ft_bucketed_random_projection_lsh,ml_pipeline)
S3method(ft_bucketed_random_projection_lsh,spark_connection)
S3method(ft_bucketed_random_projection_lsh,tbl_spark)
S3method(ft_bucketizer,ml_pipeline)
S3method(ft_bucketizer,spark_connection)
S3method(ft_bucketizer,tbl_spark)
S3method(ft_chisq_selector,ml_pipeline)
S3method(ft_chisq_selector,spark_connection)
S3method(ft_chisq_selector,tbl_spark)
S3method(ft_count_vectorizer,ml_pipeline)
S3method(ft_count_vectorizer,spark_connection)
S3method(ft_count_vectorizer,tbl_spark)
S3method(ft_dct,ml_pipeline)
S3method(ft_dct,spark_connection)
S3method(ft_dct,tbl_spark)
S3method(ft_dplyr_transformer,ml_pipeline)
S3method(ft_dplyr_transformer,spark_connection)
S3method(ft_dplyr_transformer,tbl_spark)
S3method(ft_elementwise_product,ml_pipeline)
S3method(ft_elementwise_product,spark_connection)
S3method(ft_elementwise_product,tbl_spark)
S3method(ft_feature_hasher,ml_pipeline)
S3method(ft_feature_hasher,spark_connection)
S3method(ft_feature_hasher,tbl_spark)
S3method(ft_hashing_tf,ml_pipeline)
S3method(ft_hashing_tf,spark_connection)
S3method(ft_hashing_tf,tbl_spark)
S3method(ft_idf,ml_pipeline)
S3method(ft_idf,spark_connection)
S3method(ft_idf,tbl_spark)
S3method(ft_imputer,ml_pipeline)
S3method(ft_imputer,spark_connection)
S3method(ft_imputer,tbl_spark)
S3method(ft_index_to_string,ml_pipeline)
S3method(ft_index_to_string,spark_connection)
S3method(ft_index_to_string,tbl_spark)
S3method(ft_interaction,ml_pipeline)
S3method(ft_interaction,spark_connection)
S3method(ft_interaction,tbl_spark)
S3method(ft_max_abs_scaler,ml_pipeline)
S3method(ft_max_abs_scaler,spark_connection)
S3method(ft_max_abs_scaler,tbl_spark)
S3method(ft_min_max_scaler,ml_pipeline)
S3method(ft_min_max_scaler,spark_connection)
S3method(ft_min_max_scaler,tbl_spark)
S3method(ft_minhash_lsh,ml_pipeline)
S3method(ft_minhash_lsh,spark_connection)
S3method(ft_minhash_lsh,tbl_spark)
S3method(ft_ngram,ml_pipeline)
S3method(ft_ngram,spark_connection)
S3method(ft_ngram,tbl_spark)
S3method(ft_normalizer,ml_pipeline)
S3method(ft_normalizer,spark_connection)
S3method(ft_normalizer,tbl_spark)
S3method(ft_one_hot_encoder,ml_pipeline)
S3method(ft_one_hot_encoder,spark_connection)
S3method(ft_one_hot_encoder,tbl_spark)
S3method(ft_one_hot_encoder_estimator,ml_pipeline)
S3method(ft_one_hot_encoder_estimator,spark_connection)
S3method(ft_one_hot_encoder_estimator,tbl_spark)
S3method(ft_pca,ml_pipeline)
S3method(ft_pca,spark_connection)
S3method(ft_pca,tbl_spark)
S3method(ft_polynomial_expansion,ml_pipeline)
S3method(ft_polynomial_expansion,spark_connection)
S3method(ft_polynomial_expansion,tbl_spark)
S3method(ft_quantile_discretizer,ml_pipeline)
S3method(ft_quantile_discretizer,spark_connection)
S3method(ft_quantile_discretizer,tbl_spark)
S3method(ft_r_formula,ml_pipeline)
S3method(ft_r_formula,spark_connection)
S3method(ft_r_formula,tbl_spark)
S3method(ft_regex_tokenizer,ml_pipeline)
S3method(ft_regex_tokenizer,spark_connection)
S3method(ft_regex_tokenizer,tbl_spark)
S3method(ft_robust_scaler,ml_pipeline)
S3method(ft_robust_scaler,spark_connection)
S3method(ft_robust_scaler,tbl_spark)
S3method(ft_sql_transformer,ml_pipeline)
S3method(ft_sql_transformer,spark_connection)
S3method(ft_sql_transformer,tbl_spark)
S3method(ft_standard_scaler,ml_pipeline)
S3method(ft_standard_scaler,spark_connection)
S3method(ft_standard_scaler,tbl_spark)
S3method(ft_stop_words_remover,ml_pipeline)
S3method(ft_stop_words_remover,spark_connection)
S3method(ft_stop_words_remover,tbl_spark)
S3method(ft_string_indexer,ml_pipeline)
S3method(ft_string_indexer,spark_connection)
S3method(ft_string_indexer,tbl_spark)
S3method(ft_string_indexer_model,ml_pipeline)
S3method(ft_string_indexer_model,spark_connection)
S3method(ft_string_indexer_model,tbl_spark)
S3method(ft_tokenizer,ml_pipeline)
S3method(ft_tokenizer,spark_connection)
S3method(ft_tokenizer,tbl_spark)
S3method(ft_vector_assembler,ml_pipeline)
S3method(ft_vector_assembler,spark_connection)
S3method(ft_vector_assembler,tbl_spark)
S3method(ft_vector_indexer,ml_pipeline)
S3method(ft_vector_indexer,spark_connection)
S3method(ft_vector_indexer,tbl_spark)
S3method(ft_vector_slicer,ml_pipeline)
S3method(ft_vector_slicer,spark_connection)
S3method(ft_vector_slicer,tbl_spark)
S3method(ft_word2vec,ml_pipeline)
S3method(ft_word2vec,spark_connection)
S3method(ft_word2vec,tbl_spark)
S3method(full_join,tbl_spark)
S3method(glance,ml_model)
S3method(glance,ml_model_aft_survival_regression)
S3method(glance,ml_model_als)
S3method(glance,ml_model_bisecting_kmeans)
S3method(glance,ml_model_decision_tree_classification)
S3method(glance,ml_model_decision_tree_regression)
S3method(glance,ml_model_gaussian_mixture)
S3method(glance,ml_model_gbt_classification)
S3method(glance,ml_model_gbt_regression)
S3method(glance,ml_model_generalized_linear_regression)
S3method(glance,ml_model_isotonic_regression)
S3method(glance,ml_model_kmeans)
S3method(glance,ml_model_lda)
S3method(glance,ml_model_linear_regression)
S3method(glance,ml_model_linear_svc)
S3method(glance,ml_model_logistic_regression)
S3method(glance,ml_model_multilayer_perceptron_classification)
S3method(glance,ml_model_naive_bayes)
S3method(glance,ml_model_pca)
S3method(glance,ml_model_random_forest_classification)
S3method(glance,ml_model_random_forest_regression)
S3method(head,tbl_spark)
S3method(hive_context,spark_connection)
S3method(initialize_connection,livy_connection)
S3method(initialize_connection,spark_shell_connection)
S3method(initialize_connection,test_connection)
S3method(initialize_method,default)
S3method(initialize_method,qubole)
S3method(inner_join,tbl_spark)
S3method(invoke,livy_jobj)
S3method(invoke,shell_jobj)
S3method(invoke,test_jobj)
S3method(invoke_method,spark_gateway_connection)
S3method(invoke_method,spark_shell_connection)
S3method(invoke_new,livy_connection)
S3method(invoke_new,spark_shell_connection)
S3method(invoke_new,test_connection)
S3method(invoke_static,livy_connection)
S3method(invoke_static,spark_shell_connection)
S3method(invoke_static,test_connection)
S3method(is_required_spark,spark_connection)
S3method(is_required_spark,spark_jobj)
S3method(j_invoke,livy_jobj)
S3method(j_invoke,shell_jobj)
S3method(j_invoke,test_jobj)
S3method(j_invoke_method,spark_gateway_connection)
S3method(j_invoke_method,spark_shell_connection)
S3method(j_invoke_new,livy_connection)
S3method(j_invoke_new,spark_shell_connection)
S3method(j_invoke_new,test_connection)
S3method(j_invoke_static,livy_connection)
S3method(j_invoke_static,spark_shell_connection)
S3method(j_invoke_static,test_connection)
S3method(jobj_subclass,livy_backend)
S3method(jobj_subclass,shell_backend)
S3method(jobj_subclass,spark_worker_connection)
S3method(left_join,tbl_spark)
S3method(ml_aft_survival_regression,ml_pipeline)
S3method(ml_aft_survival_regression,spark_connection)
S3method(ml_aft_survival_regression,tbl_spark)
S3method(ml_als,ml_pipeline)
S3method(ml_als,spark_connection)
S3method(ml_als,tbl_spark)
S3method(ml_binary_classification_evaluator,spark_connection)
S3method(ml_binary_classification_evaluator,tbl_spark)
S3method(ml_bisecting_kmeans,ml_pipeline)
S3method(ml_bisecting_kmeans,spark_connection)
S3method(ml_bisecting_kmeans,tbl_spark)
S3method(ml_clustering_evaluator,spark_connection)
S3method(ml_clustering_evaluator,tbl_spark)
S3method(ml_cross_validator,ml_pipeline)
S3method(ml_cross_validator,spark_connection)
S3method(ml_cross_validator,tbl_spark)
S3method(ml_decision_tree_classifier,ml_pipeline)
S3method(ml_decision_tree_classifier,spark_connection)
S3method(ml_decision_tree_classifier,tbl_spark)
S3method(ml_decision_tree_regressor,ml_pipeline)
S3method(ml_decision_tree_regressor,spark_connection)
S3method(ml_decision_tree_regressor,tbl_spark)
S3method(ml_evaluate,default)
S3method(ml_evaluate,ml_evaluator)
S3method(ml_evaluate,ml_generalized_linear_regression_model)
S3method(ml_evaluate,ml_linear_regression_model)
S3method(ml_evaluate,ml_logistic_regression_model)
S3method(ml_evaluate,ml_model_classification)
S3method(ml_evaluate,ml_model_clustering)
S3method(ml_evaluate,ml_model_generalized_linear_regression)
S3method(ml_evaluate,ml_model_linear_regression)
S3method(ml_evaluate,ml_model_logistic_regression)
S3method(ml_feature_importances,ml_model)
S3method(ml_feature_importances,ml_prediction_model)
S3method(ml_fit,default)
S3method(ml_fpgrowth,ml_pipeline)
S3method(ml_fpgrowth,spark_connection)
S3method(ml_fpgrowth,tbl_spark)
S3method(ml_gaussian_mixture,ml_pipeline)
S3method(ml_gaussian_mixture,spark_connection)
S3method(ml_gaussian_mixture,tbl_spark)
S3method(ml_gbt_classifier,ml_pipeline)
S3method(ml_gbt_classifier,spark_connection)
S3method(ml_gbt_classifier,tbl_spark)
S3method(ml_gbt_regressor,ml_pipeline)
S3method(ml_gbt_regressor,spark_connection)
S3method(ml_gbt_regressor,tbl_spark)
S3method(ml_generalized_linear_regression,ml_pipeline)
S3method(ml_generalized_linear_regression,spark_connection)
S3method(ml_generalized_linear_regression,tbl_spark)
S3method(ml_is_set,ml_pipeline_stage)
S3method(ml_is_set,spark_jobj)
S3method(ml_isotonic_regression,ml_pipeline)
S3method(ml_isotonic_regression,spark_connection)
S3method(ml_isotonic_regression,tbl_spark)
S3method(ml_kmeans,ml_pipeline)
S3method(ml_kmeans,spark_connection)
S3method(ml_kmeans,tbl_spark)
S3method(ml_lda,ml_pipeline)
S3method(ml_lda,spark_connection)
S3method(ml_lda,tbl_spark)
S3method(ml_linear_regression,ml_pipeline)
S3method(ml_linear_regression,spark_connection)
S3method(ml_linear_regression,tbl_spark)
S3method(ml_linear_svc,ml_pipeline)
S3method(ml_linear_svc,spark_connection)
S3method(ml_linear_svc,tbl_spark)
S3method(ml_logistic_regression,ml_pipeline)
S3method(ml_logistic_regression,spark_connection)
S3method(ml_logistic_regression,tbl_spark)
S3method(ml_multiclass_classification_evaluator,spark_connection)
S3method(ml_multiclass_classification_evaluator,tbl_spark)
S3method(ml_multilayer_perceptron_classifier,ml_pipeline)
S3method(ml_multilayer_perceptron_classifier,spark_connection)
S3method(ml_multilayer_perceptron_classifier,tbl_spark)
S3method(ml_naive_bayes,ml_pipeline)
S3method(ml_naive_bayes,spark_connection)
S3method(ml_naive_bayes,tbl_spark)
S3method(ml_one_vs_rest,ml_pipeline)
S3method(ml_one_vs_rest,spark_connection)
S3method(ml_one_vs_rest,tbl_spark)
S3method(ml_pipeline,ml_pipeline_stage)
S3method(ml_pipeline,spark_connection)
S3method(ml_power_iteration,spark_connection)
S3method(ml_power_iteration,tbl_spark)
S3method(ml_predict,default)
S3method(ml_predict,ml_model_classification)
S3method(ml_predict,ml_model_clustering)
S3method(ml_predict,ml_model_recommendation)
S3method(ml_predict,ml_model_regression)
S3method(ml_prefixspan,spark_connection)
S3method(ml_random_forest_classifier,ml_pipeline)
S3method(ml_random_forest_classifier,spark_connection)
S3method(ml_random_forest_classifier,tbl_spark)
S3method(ml_random_forest_regressor,ml_pipeline)
S3method(ml_random_forest_regressor,spark_connection)
S3method(ml_random_forest_regressor,tbl_spark)
S3method(ml_regression_evaluator,spark_connection)
S3method(ml_regression_evaluator,tbl_spark)
S3method(ml_save,default)
S3method(ml_save,ml_model)
S3method(ml_train_validation_split,ml_pipeline)
S3method(ml_train_validation_split,spark_connection)
S3method(ml_train_validation_split,tbl_spark)
S3method(ml_transform,default)
S3method(ml_transform,list)
S3method(ml_transform,ml_transformer)
S3method(mutate,tbl_spark)
S3method(na.fail,spark_jobj)
S3method(na.fail,tbl_spark)
S3method(na.omit,spark_jobj)
S3method(na.omit,tbl_spark)
S3method(na.replace,spark_jobj)
S3method(na.replace,tbl_spark)
S3method(nest,tbl_spark)
S3method(op_vars,lazy_sample_query)
S3method(op_vars,tbl_spark)
S3method(params_validator,ml_estimator)
S3method(params_validator,ml_generalized_linear_regression)
S3method(params_validator,pre_ml_estimator)
S3method(pivot_longer,tbl_spark)
S3method(pivot_wider,tbl_spark)
S3method(post_ml_obj,ml_pipeline)
S3method(post_ml_obj,spark_connection)
S3method(post_ml_obj,tbl_spark)
S3method(predict,ml_model_classification)
S3method(predict,ml_model_regression)
S3method(print,ml_cross_validator)
S3method(print,ml_cross_validator_model)
S3method(print,ml_estimator)
S3method(print,ml_evaluator)
S3method(print,ml_model)
S3method(print,ml_model_aft_survival_regression)
S3method(print,ml_model_bisecting_kmeans)
S3method(print,ml_model_generalized_linear_regression)
S3method(print,ml_model_kmeans)
S3method(print,ml_model_linear_regression)
S3method(print,ml_model_linear_svc)
S3method(print,ml_model_logistic_regression)
S3method(print,ml_model_naive_bayes)
S3method(print,ml_model_pca)
S3method(print,ml_pipeline)
S3method(print,ml_pipeline_model)
S3method(print,ml_summary)
S3method(print,ml_train_validation_split)
S3method(print,ml_train_validation_split_model)
S3method(print,ml_transformer)
S3method(print,spark_jobj)
S3method(print,spark_log)
S3method(print,spark_stream)
S3method(print,spark_web_url)
S3method(print,src_spark)
S3method(print_jobj,livy_connection)
S3method(print_jobj,spark_gateway_connection)
S3method(print_jobj,spark_shell_connection)
S3method(print_jobj,test_connection)
S3method(quote_sql_name,character)
S3method(quote_sql_name,name)
S3method(rbind,tbl_spark)
S3method(read_bin,default)
S3method(read_bin,livy_backend)
S3method(read_bin,spark_worker_connection)
S3method(replace_na,spark_jobj)
S3method(replace_na,tbl_spark)
S3method(residuals,ml_model)
S3method(residuals,ml_model_generalized_linear_regression)
S3method(residuals,ml_model_linear_regression)
S3method(right_join,tbl_spark)
S3method(same_src,src_spark)
S3method(sample_frac,tbl_spark)
S3method(sample_n,tbl_spark)
S3method(sdf_copy_to,default)
S3method(sdf_copy_to,test_connection)
S3method(sdf_distinct,spark_jobj)
S3method(sdf_distinct,tbl_spark)
S3method(sdf_import,default)
S3method(sdf_import,test_connection)
S3method(sdf_predict,default)
S3method(sdf_read_column,spark_jobj)
S3method(sdf_read_column,tbl_spark)
S3method(sdf_register,list)
S3method(sdf_register,spark_jobj)
S3method(sdf_register,tbl_spark)
S3method(sdf_remote_name,default)
S3method(sdf_remote_name,tbl_spark)
S3method(sdf_residuals,ml_model_generalized_linear_regression)
S3method(sdf_residuals,ml_model_linear_regression)
S3method(sdf_schema,default)
S3method(sdf_schema,tbl_spark)
S3method(select,tbl_spark)
S3method(separate,tbl_spark)
S3method(spark_apply,default)
S3method(spark_connect_method,default)
S3method(spark_connection,default)
S3method(spark_connection,ml_pipeline)
S3method(spark_connection,ml_pipeline_model)
S3method(spark_connection,ml_pipeline_stage)
S3method(spark_connection,spark_connection)
S3method(spark_connection,spark_jobj)
S3method(spark_connection,src_spark)
S3method(spark_connection,tbl_spark)
S3method(spark_dataframe,default)
S3method(spark_dataframe,spark_connection)
S3method(spark_dataframe,spark_jobj)
S3method(spark_dataframe,tbl_spark)
S3method(spark_disconnect,character)
S3method(spark_disconnect,livy_connection)
S3method(spark_disconnect,spark_connection)
S3method(spark_disconnect,spark_shell_connection)
S3method(spark_disconnect,test_connection)
S3method(spark_ide_columns,spark_connection)
S3method(spark_ide_connection_actions,default)
S3method(spark_ide_connection_closed,default)
S3method(spark_ide_connection_open,default)
S3method(spark_ide_connection_updated,default)
S3method(spark_ide_objects,spark_connection)
S3method(spark_ide_preview,spark_connection)
S3method(spark_insert_table,spark_jobj)
S3method(spark_insert_table,tbl_spark)
S3method(spark_integ_test_skip,default)
S3method(spark_jobj,default)
S3method(spark_jobj,ml_evaluator)
S3method(spark_jobj,ml_model)
S3method(spark_jobj,ml_pipeline_stage)
S3method(spark_jobj,spark_jobj)
S3method(spark_log,default)
S3method(spark_log,livy_connection)
S3method(spark_log,spark_gateway_connection)
S3method(spark_log,spark_shell_connection)
S3method(spark_log,test_connection)
S3method(spark_read_csv,spark_connection)
S3method(spark_read_json,spark_connection)
S3method(spark_read_orc,spark_connection)
S3method(spark_read_parquet,spark_connection)
S3method(spark_read_text,spark_connection)
S3method(spark_session,spark_connection)
S3method(spark_version,databricks_connection)
S3method(spark_version,default)
S3method(spark_version,synapse_connection)
S3method(spark_web,default)
S3method(spark_web,livy_connection)
S3method(spark_web,spark_gateway_connection)
S3method(spark_web,test_connection)
S3method(spark_write,spark_jobj)
S3method(spark_write,tbl_spark)
S3method(spark_write_csv,spark_jobj)
S3method(spark_write_csv,tbl_spark)
S3method(spark_write_delta,spark_jobj)
S3method(spark_write_delta,tbl_spark)
S3method(spark_write_jdbc,spark_jobj)
S3method(spark_write_jdbc,tbl_spark)
S3method(spark_write_json,spark_jobj)
S3method(spark_write_json,tbl_spark)
S3method(spark_write_orc,spark_jobj)
S3method(spark_write_orc,tbl_spark)
S3method(spark_write_parquet,spark_jobj)
S3method(spark_write_parquet,tbl_spark)
S3method(spark_write_source,spark_jobj)
S3method(spark_write_source,tbl_spark)
S3method(spark_write_table,spark_jobj)
S3method(spark_write_table,tbl_spark)
S3method(spark_write_text,spark_jobj)
S3method(spark_write_text,tbl_spark)
S3method(sql_build,lazy_sample_query)
S3method(sql_escape_ident,spark_connection)
S3method(sql_query_explain,spark_connection)
S3method(sql_query_fields,spark_connection)
S3method(sql_query_save,spark_connection)
S3method(sql_query_set_op,spark_connection)
S3method(sql_table_analyze,spark_connection)
S3method(sql_translation,spark_connection)
S3method(src_tbls,spark_connection)
S3method(stream_trigger_create,stream_trigger_continuous)
S3method(stream_trigger_create,stream_trigger_interval)
S3method(summarise,tbl_spark)
S3method(summary,ml_cross_validator_model)
S3method(summary,ml_model_generalized_linear_regression)
S3method(summary,ml_model_linear_regression)
S3method(summary,ml_model_logistic_regression)
S3method(summary,ml_model_naive_bayes)
S3method(summary,ml_train_validation_split_model)
S3method(tbl,spark_connection)
S3method(tbl,src_spark)
S3method(tbl_ptype,tbl_spark)
S3method(tbl_vars,spark_jobj)
S3method(tbl_vars,tbl_spark)
S3method(tidy,ml_model)
S3method(tidy,ml_model_aft_survival_regression)
S3method(tidy,ml_model_als)
S3method(tidy,ml_model_bisecting_kmeans)
S3method(tidy,ml_model_decision_tree_classification)
S3method(tidy,ml_model_decision_tree_regression)
S3method(tidy,ml_model_gaussian_mixture)
S3method(tidy,ml_model_gbt_classification)
S3method(tidy,ml_model_gbt_regression)
S3method(tidy,ml_model_generalized_linear_regression)
S3method(tidy,ml_model_isotonic_regression)
S3method(tidy,ml_model_kmeans)
S3method(tidy,ml_model_lda)
S3method(tidy,ml_model_linear_regression)
S3method(tidy,ml_model_linear_svc)
S3method(tidy,ml_model_logistic_regression)
S3method(tidy,ml_model_multilayer_perceptron_classification)
S3method(tidy,ml_model_naive_bayes)
S3method(tidy,ml_model_pca)
S3method(tidy,ml_model_random_forest_classification)
S3method(tidy,ml_model_random_forest_regression)
S3method(tidyselect_data_has_predicates,tbl_spark)
S3method(tidyselect_data_proxy,tbl_spark)
S3method(transmute,tbl_spark)
S3method(type_sum,spark_jobj)
S3method(unite,tbl_spark)
S3method(unnest,tbl_spark)
S3method(worker_invoke,shell_jobj)
export("%->%")
export("%>%")
export(arrow_enabled_object)
export(augment)
export(collect)
export(collect_from_rds)
export(compile_package_jars)
export(connection_config)
export(connection_is_open)
export(connection_spark_shinyapp)
export(copy_to)
export(distinct)
export(download_scalac)
export(fill)
export(filter)
export(find_scalac)
export(ft_binarizer)
export(ft_bucketed_random_projection_lsh)
export(ft_bucketizer)
export(ft_chisq_selector)
export(ft_count_vectorizer)
export(ft_dct)
export(ft_discrete_cosine_transform)
export(ft_dplyr_transformer)
export(ft_elementwise_product)
export(ft_feature_hasher)
export(ft_hashing_tf)
export(ft_idf)
export(ft_imputer)
export(ft_index_to_string)
export(ft_interaction)
export(ft_max_abs_scaler)
export(ft_min_max_scaler)
export(ft_minhash_lsh)
export(ft_ngram)
export(ft_normalizer)
export(ft_one_hot_encoder)
export(ft_one_hot_encoder_estimator)
export(ft_pca)
export(ft_polynomial_expansion)
export(ft_quantile_discretizer)
export(ft_r_formula)
export(ft_regex_tokenizer)
export(ft_robust_scaler)
export(ft_sql_transformer)
export(ft_standard_scaler)
export(ft_stop_words_remover)
export(ft_string_indexer)
export(ft_string_indexer_model)
export(ft_tokenizer)
export(ft_vector_assembler)
export(ft_vector_indexer)
export(ft_vector_slicer)
export(ft_word2vec)
export(full_join)
export(get_spark_sql_catalog_implementation)
export(glance)
export(hive_context)
export(hive_context_config)
export(hof_aggregate)
export(hof_array_sort)
export(hof_exists)
export(hof_filter)
export(hof_forall)
export(hof_map_filter)
export(hof_map_zip_with)
export(hof_transform)
export(hof_transform_keys)
export(hof_transform_values)
export(hof_zip_with)
export(inner_join)
export(invoke)
export(invoke_method)
export(invoke_new)
export(invoke_static)
export(is_ml_estimator)
export(is_ml_transformer)
export(j_invoke)
export(j_invoke_method)
export(j_invoke_new)
export(j_invoke_static)
export(jarray)
export(java_context)
export(jfloat)
export(jfloat_array)
export(jobj_class)
export(jobj_set_param)
export(left_join)
export(livy_available_versions)
export(livy_config)
export(livy_home_dir)
export(livy_install)
export(livy_install_dir)
export(livy_installed_versions)
export(livy_service_start)
export(livy_service_stop)
export(ml_add_stage)
export(ml_aft_survival_regression)
export(ml_als)
export(ml_approx_nearest_neighbors)
export(ml_approx_similarity_join)
export(ml_association_rules)
export(ml_binary_classification_eval)
export(ml_binary_classification_evaluator)
export(ml_bisecting_kmeans)
export(ml_call_constructor)
export(ml_chisquare_test)
export(ml_classification_eval)
export(ml_clustering_evaluator)
export(ml_clustering_pipeline)
export(ml_compute_cost)
export(ml_compute_silhouette_measure)
export(ml_construct_model_clustering)
export(ml_construct_model_supervised)
export(ml_corr)
export(ml_cross_validator)
export(ml_decision_tree)
export(ml_decision_tree_classifier)
export(ml_decision_tree_regressor)
export(ml_default_stop_words)
export(ml_describe_topics)
export(ml_evaluate)
export(ml_feature_importances)
export(ml_find_synonyms)
export(ml_fit)
export(ml_fit_and_transform)
export(ml_fpgrowth)
export(ml_freq_itemsets)
export(ml_freq_seq_patterns)
export(ml_gaussian_mixture)
export(ml_gbt_classifier)
export(ml_gbt_regressor)
export(ml_generalized_linear_regression)
export(ml_gradient_boosted_trees)
export(ml_is_set)
export(ml_isotonic_regression)
export(ml_kmeans)
export(ml_labels)
export(ml_lda)
export(ml_linear_regression)
export(ml_linear_svc)
export(ml_load)
export(ml_log_likelihood)
export(ml_log_perplexity)
export(ml_logistic_regression)
export(ml_metrics_binary)
export(ml_metrics_multiclass)
export(ml_metrics_regression)
export(ml_model_data)
export(ml_multiclass_classification_evaluator)
export(ml_multilayer_perceptron)
export(ml_multilayer_perceptron_classifier)
export(ml_naive_bayes)
export(ml_one_vs_rest)
export(ml_param)
export(ml_param_map)
export(ml_params)
export(ml_pca)
export(ml_pipeline)
export(ml_power_iteration)
export(ml_predict)
export(ml_prefixspan)
export(ml_random_forest)
export(ml_random_forest_classifier)
export(ml_random_forest_regressor)
export(ml_recommend)
export(ml_regression_evaluator)
export(ml_save)
export(ml_stage)
export(ml_stages)
export(ml_standardize_formula)
export(ml_sub_models)
export(ml_summary)
export(ml_supervised_pipeline)
export(ml_survival_regression)
export(ml_topics_matrix)
export(ml_train_validation_split)
export(ml_transform)
export(ml_tree_feature_importance)
export(ml_uid)
export(ml_validation_metrics)
export(ml_vocabulary)
export(mutate)
export(na.replace)
export(nest)
export(new_ml_classification_model)
export(new_ml_classifier)
export(new_ml_clustering_model)
export(new_ml_estimator)
export(new_ml_model)
export(new_ml_model_classification)
export(new_ml_model_clustering)
export(new_ml_model_prediction)
export(new_ml_model_regression)
export(new_ml_prediction_model)
export(new_ml_predictor)
export(new_ml_probabilistic_classification_model)
export(new_ml_probabilistic_classifier)
export(new_ml_transformer)
export(pivot_longer)
export(pivot_wider)
export(print_jobj)
export(quote_sql_name)
export(random_string)
export(reactiveSpark)
export(registerDoSpark)
export(register_extension)
export(registered_extensions)
export(replace_na)
export(right_join)
export(sdf_along)
export(sdf_bind_cols)
export(sdf_bind_rows)
export(sdf_broadcast)
export(sdf_checkpoint)
export(sdf_coalesce)
export(sdf_collect)
export(sdf_copy_to)
export(sdf_crosstab)
export(sdf_debug_string)
export(sdf_describe)
export(sdf_dim)
export(sdf_distinct)
export(sdf_drop_duplicates)
export(sdf_expand_grid)
export(sdf_fit)
export(sdf_fit_and_transform)
export(sdf_from_avro)
export(sdf_import)
export(sdf_is_streaming)
export(sdf_last_index)
export(sdf_len)
export(sdf_load_parquet)
export(sdf_load_table)
export(sdf_ncol)
export(sdf_nrow)
export(sdf_num_partitions)
export(sdf_partition)
export(sdf_partition_sizes)
export(sdf_persist)
export(sdf_pivot)
export(sdf_predict)
export(sdf_project)
export(sdf_quantile)
export(sdf_random_split)
export(sdf_rbeta)
export(sdf_rbinom)
export(sdf_rcauchy)
export(sdf_rchisq)
export(sdf_read_column)
export(sdf_register)
export(sdf_repartition)
export(sdf_residuals)
export(sdf_rexp)
export(sdf_rgamma)
export(sdf_rgeom)
export(sdf_rhyper)
export(sdf_rlnorm)
export(sdf_rnorm)
export(sdf_rpois)
export(sdf_rt)
export(sdf_runif)
export(sdf_rweibull)
export(sdf_sample)
export(sdf_save_parquet)
export(sdf_save_table)
export(sdf_schema)
export(sdf_separate_column)
export(sdf_seq)
export(sdf_sort)
export(sdf_sql)
export(sdf_to_avro)
export(sdf_transform)
export(sdf_unnest_longer)
export(sdf_unnest_wider)
export(sdf_weighted_sample)
export(sdf_with_sequential_id)
export(sdf_with_unique_id)
export(select)
export(separate)
export(slice_.tbl_spark)
export(spark_adaptive_query_execution)
export(spark_advisory_shuffle_partition_size)
export(spark_apply)
export(spark_apply_bundle)
export(spark_apply_log)
export(spark_auto_broadcast_join_threshold)
export(spark_available_versions)
export(spark_coalesce_initial_num_partitions)
export(spark_coalesce_min_num_partitions)
export(spark_coalesce_shuffle_partitions)
export(spark_compilation_spec)
export(spark_compile)
export(spark_config)
export(spark_config_exists)
export(spark_config_kubernetes)
export(spark_config_packages)
export(spark_config_settings)
export(spark_config_value)
export(spark_connect)
export(spark_connect_method)
export(spark_connection)
export(spark_connection_find)
export(spark_connection_is_open)
export(spark_context)
export(spark_context_config)
export(spark_dataframe)
export(spark_default_compilation_spec)
export(spark_default_version)
export(spark_dependency)
export(spark_dependency_fallback)
export(spark_disconnect)
export(spark_disconnect_all)
export(spark_extension)
export(spark_get_checkpoint_dir)
export(spark_get_java)
export(spark_home_dir)
export(spark_home_set)
export(spark_ide_columns)
export(spark_ide_connection_actions)
export(spark_ide_connection_closed)
export(spark_ide_connection_open)
export(spark_ide_connection_updated)
export(spark_ide_objects)
export(spark_ide_preview)
export(spark_insert_table)
export(spark_install)
export(spark_install_dir)
export(spark_install_find)
export(spark_install_tar)
export(spark_installed_versions)
export(spark_integ_test_skip)
export(spark_jobj)
export(spark_last_error)
export(spark_load_table)
export(spark_log)
export(spark_pipeline_stage)
export(spark_read)
export(spark_read_avro)
export(spark_read_binary)
export(spark_read_csv)
export(spark_read_delta)
export(spark_read_image)
export(spark_read_jdbc)
export(spark_read_json)
export(spark_read_libsvm)
export(spark_read_orc)
export(spark_read_parquet)
export(spark_read_source)
export(spark_read_table)
export(spark_read_text)
export(spark_save_table)
export(spark_session)
export(spark_session_config)
export(spark_set_checkpoint_dir)
export(spark_submit)
export(spark_table_name)
export(spark_uninstall)
export(spark_version)
export(spark_version_from_home)
export(spark_versions)
export(spark_web)
export(spark_write)
export(spark_write_avro)
export(spark_write_csv)
export(spark_write_delta)
export(spark_write_jdbc)
export(spark_write_json)
export(spark_write_orc)
export(spark_write_parquet)
export(spark_write_rds)
export(spark_write_source)
export(spark_write_table)
export(spark_write_text)
export(sparklyr_get_backend_port)
export(src_databases)
export(stream_find)
export(stream_generate_test)
export(stream_id)
export(stream_lag)
export(stream_name)
export(stream_read_cloudfiles)
export(stream_read_csv)
export(stream_read_delta)
export(stream_read_json)
export(stream_read_kafka)
export(stream_read_orc)
export(stream_read_parquet)
export(stream_read_socket)
export(stream_read_table)
export(stream_read_text)
export(stream_render)
export(stream_stats)
export(stream_stop)
export(stream_trigger_continuous)
export(stream_trigger_interval)
export(stream_view)
export(stream_watermark)
export(stream_write_console)
export(stream_write_csv)
export(stream_write_delta)
export(stream_write_json)
export(stream_write_kafka)
export(stream_write_memory)
export(stream_write_orc)
export(stream_write_parquet)
export(stream_write_table)
export(stream_write_text)
export(tbl_cache)
export(tbl_change_db)
export(tbl_uncache)
export(tidy)
export(tune_grid_spark)
export(unite)
export(unnest)
export(worker_spark_apply_unbundle)
exportClasses(DBISparkResult)
exportClasses(spark_connection)
exportClasses(spark_jobj)
import(DBI)
import(httr)
import(methods)
import(utils)
importFrom(DBI,dbExecute)
importFrom(DBI,dbGetQuery)
importFrom(DBI,dbListTables)
importFrom(DBI,dbSendQuery)
importFrom(dbplyr,build_sql)
importFrom(dbplyr,db_connection_describe)
importFrom(dbplyr,dbplyr_edition)
importFrom(dbplyr,escape)
importFrom(dbplyr,ident)
importFrom(dbplyr,op_vars)
importFrom(dbplyr,sql)
importFrom(dbplyr,sql_build)
importFrom(dbplyr,sql_query_explain)
importFrom(dbplyr,sql_query_fields)
importFrom(dbplyr,sql_query_save)
importFrom(dbplyr,sql_query_set_op)
importFrom(dbplyr,sql_quote)
importFrom(dbplyr,sql_render)
importFrom(dbplyr,sql_table_analyze)
importFrom(dbplyr,sql_translation)
importFrom(dbplyr,tbl_sql)
importFrom(dbplyr,win_current_group)
importFrom(dbplyr,win_current_order)
importFrom(dbplyr,win_over)
importFrom(dplyr,"%>%")
importFrom(dplyr,.data)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,collect)
importFrom(dplyr,compute)
importFrom(dplyr,copy_to)
importFrom(dplyr,count)
importFrom(dplyr,db_data_type)
importFrom(dplyr,distinct)
importFrom(dplyr,do)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,groups)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,right_join)
importFrom(dplyr,same_src)
importFrom(dplyr,sample_frac)
importFrom(dplyr,sample_n)
importFrom(dplyr,select)
importFrom(dplyr,slice_)
importFrom(dplyr,sql_escape_ident)
importFrom(dplyr,src_tbls)
importFrom(dplyr,summarise)
importFrom(dplyr,tbl)
importFrom(dplyr,tbl_ptype)
importFrom(dplyr,tbl_vars)
importFrom(dplyr,tibble)
importFrom(dplyr,transmute)
importFrom(dplyr,type_sum)
importFrom(dplyr,ungroup)
importFrom(generics,augment)
importFrom(generics,glance)
importFrom(generics,tidy)
importFrom(httr,GET)
importFrom(httr,POST)
importFrom(httr,http_error)
importFrom(httr,http_status)
importFrom(httr,text_content)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(openssl,base64_decode)
importFrom(openssl,base64_encode)
importFrom(openssl,multihash)
importFrom(openssl,rand_num)
importFrom(purrr,imap)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dfr)
importFrom(purrr,reduce)
importFrom(purrr,transpose)
importFrom(rlang,"!!")
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,`!!`)
importFrom(rlang,abort)
importFrom(rlang,as_label)
importFrom(rlang,as_name)
importFrom(rlang,check_dots_empty)
importFrom(rlang,check_dots_used)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,quo)
importFrom(rlang,set_names)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(rstudioapi,sendToConsole)
importFrom(rstudioapi,showQuestion)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,gaussian)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(tidyr,fill)
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,separate)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(tidyselect,eval_select)
importFrom(tidyselect,tidyselect_data_has_predicates)
importFrom(tidyselect,tidyselect_data_proxy)
importFrom(tidyselect,vars_select)
