#' @import methods
#' @import DBI
#' @import utils
#' @importFrom stats as.formula coefficients gaussian na.fail na.omit predict quantile
#' @importFrom dplyr type_sum
#' @importFrom rlang %||%
#' @importFrom purrr map_chr
#' @importFrom openssl multihash rand_num
NULL

#' @importFrom generics tidy
#' @export
generics::tidy

#' @importFrom generics augment
#' @export
generics::augment

#' @importFrom generics glance
#' @export
generics::glance

#' @importFrom rlang check_dots_used check_dots_empty
NULL
