% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\name{pivot_wider}
\alias{pivot_wider}
\title{Pivot wider}
\description{
See \code{\link[tidyr:pivot_wider]{pivot_wider}} for more details.
}
