% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\name{mutate}
\alias{mutate}
\title{Mutate}
\description{
See \code{\link[dplyr:mutate]{mutate}} for more details.
}
