% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dplyr_spark_table.R
\name{slice_.tbl_spark}
\alias{slice_.tbl_spark}
\title{Slice (not supported)}
\usage{
slice_.tbl_spark(.data, ..., .dots)
}
\description{
Slice (not supported)
}
\keyword{internal}
