% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dplyr_hof.R
\name{dplyr_hof}
\alias{dplyr_hof}
\title{dplyr wrappers for Apache Spark higher order functions}
\description{
These methods implement dplyr grammars for Apache Spark higher order functions
}
