% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ml_pipeline_utils.R
\name{ml_uid}
\alias{ml_uid}
\title{Spark ML -- UID}
\usage{
ml_uid(x)
}
\arguments{
\item{x}{A Spark ML object}
}
\description{
Extracts the UID of an ML object.
}
