% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\name{right_join}
\alias{right_join}
\title{Right join}
\description{
See \code{\link[dplyr:mutate-joins]{right_join}} for more details.
}
