% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/connection_spark.R
\name{spark_jobj-class}
\alias{spark_jobj-class}
\title{spark_jobj class}
\description{
spark_jobj class
}
