% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\name{unnest}
\alias{unnest}
\title{Unnest}
\description{
See \code{\link[tidyr:nest]{unnest}} for more details.
}
