% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/connection_spark.R
\name{spark_connection-class}
\alias{spark_connection-class}
\title{spark_connection class}
\description{
spark_connection class
}
