% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\name{unite}
\alias{unite}
\title{Unite}
\description{
See \code{\link[tidyr:unite]{unite}} for more details.
}
