% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\name{collect}
\alias{collect}
\title{Collect}
\description{
See \code{\link[dplyr:compute]{collect}} for more details.
}
\keyword{internal}
